require(mvtnorm)
require(psych)
require(gamlss.dist)
require(gamlss)
require(pracma)
require(survival)
require(asaur)

attach(pharmacoSmoking)
survdiff(Surv(ttr, relapse) ~ grp)
zeros=which(ttr==0)
ttr2=ttr[-zeros]
relapse2=relapse[-zeros]
grp2=grp[-zeros]
age2=age[-zeros]
gender2=gender[-zeros]
race2=race[-zeros]
race3=as.factor(ifelse(race2=="white","white","non-white"))
employment2=employment[-zeros]
employment3=as.factor(ifelse(employment2=="ft","ft","non-ft"))
yearsSmoking2=yearsSmoking[-zeros]
levelSmoking2=levelSmoking[-zeros]
ageGroup22=ageGroup2[-zeros]
ageGroup42=ageGroup4[-zeros]
priorAttempts2=priorAttempts[-zeros]
longestNoSmoke2=longestNoSmoke[-zeros]



llike<-function(beta,y,x,delta,sigma=1)
{mu<-x%*%beta;-sum(delta*(-log(sigma)+(y-mu)/sigma)-exp((y-mu)/sigma))}
llike2<-function(theta,y,x,delta)
{
	sigma<-exp(theta[1])
	beta<-theta[-1]
	mu<-x%*%beta
	-sum(delta*(-log(sigma)+(y-mu)/sigma)-exp((y-mu)/sigma))
}
sigma.jackk<-function(y,x,delta)
{
	sigma.jackk<-c()
	n<-length(y);p<-ncol(x)
	for(b in 1:n)
	{
		aux.fit=nlminb(rep(0,p+1),llike2,y=y[-b],x=x[-b,],delta=delta[-b])
		sigma.jackk[b]=exp(aux.fit$par)[1]
	}
	sigma.p<-mean(sigma.jackk)
	aux.fit=nlminb(rep(0,p+1),llike2,y=y,x=x,delta=delta)
	n*exp(aux.fit$par[1])-(n-1)*sigma.p
}
llike3<-function(beta,y,x,delta,sigma)
{
	mu<-x%*%beta
	-sum(delta*(-log(sigma)+(y-mu)/sigma)-exp((y-mu)/sigma))
}


	T=ttr2
	n=length(T)
	L<-rep(182,n)
	##Percent of censoring times
	##sum(ifelse(T-L>=0,1,0))/n
	t<-c()
	for(j in 1:n)
	{
		t[j]<-min(T[j],L[j])
	}
	y<-c(log(t))
	delta=relapse2
	
	x=model.matrix(~grp2+age2+employment3)
	p=ncol(x)
	sigma=sigma.jackk(y,x,delta)
	aux.fit=nlminb(rep(0,p),llike,y=y,x=x,delta=delta,sigma=sigma,control=list(iter.max=20000,eval.max=10000))


	###Computation of skewness coefficient
	mu<-x%*%aux.fit$par
	w=1-exp(-L^(1/sigma)*exp(-mu/sigma))
	w1<--(1/sigma)*L^(1/sigma)*exp(-L^(1/sigma)*exp(-mu/sigma)-mu/sigma)
	W<-diag(c(w))
	W1<-diag(c(w1))
	P<-sigma^(2)*solve(t(x)%*%W%*%x)%*%t(x)
	Op<-matrix(1,nrow=n)
	K3<--sigma^(-3)*P^3%*%(W+3*sigma*W1)%*%Op
	K<-sigma^(-3)*diag((solve(t(x)%*%W%*%x)))^(-3/2)
	se=sqrt(diag(sigma^(2)*solve(t(x)%*%W%*%x)))



results<-round(cbind(aux.fit$par, se, K3*K),4)
colnames(results)<-c("estimate","s.e.","gamma")
rownames(results)<-paste("beta",1:p,sep="")


#########Bootstrap

b=1000
betas.boot=c()
skew.boot=c()
for(j in 1:b)
{
	u=sample(1:n,size=n,replace=TRUE)
	x.boot=x[u,]
	y.boot=y[u]
	delta.boot=delta[u]
	aux.fit=nlminb(rep(0,p),llike,y=y.boot,x=x.boot,delta=delta.boot,sigma=sigma,control=list(iter.max=20000,eval.max=10000))

	###Computation of skewness coefficient
	mu<-x.boot%*%aux.fit$par
	w=1-exp(-L^(1/sigma)*exp(-mu/sigma))
	w1<--(1/sigma)*L^(1/sigma)*exp(-L^(1/sigma)*exp(-mu/sigma)-mu/sigma)
	W<-diag(c(w))
	W1<-diag(c(w1))
	P<-sigma^(2)*solve(t(x.boot)%*%W%*%x.boot)%*%t(x.boot)
	Op<-matrix(1,nrow=n)
	K3<--sigma^(-3)*P^3%*%(W+3*sigma*W1)%*%Op
	K<-sigma^(-3)*diag((solve(t(x.boot)%*%W%*%x.boot)))^(-3/2)

	betas.boot=rbind(betas.boot,aux.fit$par)
	skew.boot=rbind(skew.boot,c(K3*K))
}


### Table 2 in the manuscript
### Skewness of maximum likelihood estimators in the
### Weibull censored data
results


#####Plot in Figure 1
### Skewness of maximum likelihood estimators in the
### Weibull censored data

bbb=c("Intercept","grp","age","employment")
dir=rep("topleft",7)
dir[4]="topright"
par(mai = c(0.85,0.97,0.30,0.05)) # Margins: inf, left, sup and right
for(j in 1:4)
{
	plot(density(betas.boot[,j]),xlab=substitute(hat(beta)[r],list(r=bbb[j])),ylab="Estimated density function",las=1,cex.lab=1.5,cex.axis=1.5,main="",lwd=2)
	abline(v=results[j,1],col="red",lwd=3,lty=2)
	curve(dnorm(x,mean=mean(betas.boot[,j]),sd=sd(betas.boot[,j])),add=TRUE,col="blue",lwd=2)
	mm=c("bootstrap","asymp. normal dist.")
	legend(dir[j],mm,lwd=2,col=c(1,"blue"),bty="n",cex=1.2)
	nombre<-paste("app2beta",j,".pdf",sep="")
	dev.copy(pdf,nombre)
	dev.off()
}


