rm(list=ls(all=TRUE))
require(moments)
llike<-function(beta,y,x,delta,sigma=1)
{mu<-x%*%beta;-sum(delta*(-log(sigma)+(y-mu)/sigma)-exp((y-mu)/sigma))}
llike2<-function(theta,y,x,delta)
{
	sigma<-exp(theta[1])
	beta<-theta[-1]
	mu<-x%*%beta
	-sum(delta*(-log(sigma)+(y-mu)/sigma)-exp((y-mu)/sigma))
}
sigma.jackk<-function(y,x,delta)
{
	sigma.jackk<-c()
	n<-length(y);p<-ncol(x)
	for(b in 1:n)
	{
		aux.fit=nlminb(rep(0,p+1),llike2,y=y[-b],x=x[-b,],delta=delta[-b])
		sigma.jackk[b]=exp(aux.fit$par)[1]
	}
	sigma.p<-mean(sigma.jackk)
	aux.fit=nlminb(rep(0,p+1),llike2,y=y,x=x,delta=delta)
	n*exp(aux.fit$par[1])-(n-1)*sigma.p
}
llike3<-function(beta,y,x,delta,sigma)
{
	mu<-x%*%beta
	-sum(delta*(-log(sigma)+(y-mu)/sigma)-exp((y-mu)/sigma))
}

T<-c(5.79, 1579.52, 2323.70, 68.85, 426.07, 110.29, 108.29, 1067.60, 
17.05, 22.66, 21.02, 175.88, 139.07,
144.12, 20.46, 43.40, 194.90, 47.30,
7.74, 0.40, 82.85, 9.88, 89.29, 215.10,
2.75, 0.79, 15.93, 3.91, 2.7,
0.69, 100.58, 27.80, 13.95, 53.24,
0.96, 4.15, 0.19, 0.78, 8.01,
31.75, 7.35, 6.50, 8.27, 33.91,
32.52, 3.16, 4.85, 2.78, 4.67,
1.31, 12.06, 36.71, 72.89,
1.97, 0.59, 2.58, 1.69, 2.71,
25.50, 0.35, 0.99, 3.99, 3.67,
2.07, 0.96, 5.35, 2.90, 13.77, 0.47, 0.73, 1.40, 0.74, 0.39,
1.13, 0.09, 2.38)
x1<-c(rep(26,3),rep(28,5),rep(30,11),rep(32,15),rep(34,19),rep(36,15),rep(38,8))
n=length(T)
L<-rep(200,n)
##Percent of censoring times
##sum(ifelse(T-L>=0,1,0))/n
t<-c();delta<-rep(1,n)
for(j in 1:n)
{
	t[j]<-min(T[j],L[j])
	if(T[j]>L[j])
	{delta[j]=0}
}
y<-c(log(t))

x=model.matrix(~x1)
p=ncol(x)
sigma=sigma.jackk(y,x,delta)
aux.fit=nlminb(rep(0,p),llike,y=y,x=x,delta=delta,sigma=sigma,control=list(iter.max=20000,eval.max=10000))

###Computation of skewness coefficient
	mu<-x%*%aux.fit$par
	w=1-exp(-L^(1/sigma)*exp(-mu/sigma))
	w1<--(1/sigma)*L^(1/sigma)*exp(-L^(1/sigma)*exp(-mu/sigma)-mu/sigma)
	W<-diag(c(w))
	W1<-diag(c(w1))
	P<-sigma^(2)*solve(t(x)%*%W%*%x)%*%t(x)
	Op<-matrix(1,nrow=n)
	K3<--sigma^(-3)*P^3%*%(W+3*sigma*W1)%*%Op
	K<-sigma^(-3)*diag((solve(t(x)%*%W%*%x)))^(-3/2)
	se=sqrt(diag(sigma^(2)*solve(t(x)%*%W%*%x)))



#########Bootstrap

b=1000
betas.boot=c()
skew.boot=c()
for(j in 1:b)
{
	u=sample(1:n,size=n,replace=TRUE)
	x.boot=x[u,]
	y.boot=y[u]
	delta.boot=delta[u]
	aux.fit=nlminb(rep(0,p),llike,y=y.boot,x=x.boot,delta=delta.boot,sigma=sigma,control=list(iter.max=20000,eval.max=10000))

	###Computation of skewness coefficient
	mu<-x.boot%*%aux.fit$par
	w=1-exp(-L^(1/sigma)*exp(-mu/sigma))
	w1<--(1/sigma)*L^(1/sigma)*exp(-L^(1/sigma)*exp(-mu/sigma)-mu/sigma)
	W<-diag(c(w))
	W1<-diag(c(w1))
	P<-sigma^(2)*solve(t(x.boot)%*%W%*%x.boot)%*%t(x.boot)
	Op<-matrix(1,nrow=n)
	K3<--sigma^(-3)*P^3%*%(W+3*sigma*W1)%*%Op
	K<-sigma^(-3)*diag((solve(t(x.boot)%*%W%*%x.boot)))^(-3/2)

	betas.boot=rbind(betas.boot,aux.fit$par)
	skew.boot=rbind(skew.boot,c(K3*K))
}


### Table 3 in the manuscript
### Skewness of maximum likelihood estimators in the
### Weibull censored data

results<-round(cbind(aux.fit$par, se, K3*K),4)
colnames(results)<-c("estimate","s.e.","gamma","K3")
rownames(results)<-paste("beta",1:p,sep="")
results


#####Plot in Figure 2
### Skewness of maximum likelihood estimators in the
### Weibull censored data

bbb=c("Intercept","log-voltage")
dir=rep("topleft",7)
dir[4]="topright"
par(mai = c(0.85,0.97,0.30,0.05)) # Margins: inf, left, sup and right
for(j in 1:2)
{
	plot(density(betas.boot[,j]),xlab=substitute(hat(beta)[r],list(r=bbb[j])),ylab="Estimated density function",las=1,cex.lab=1.5,cex.axis=1.5,main="",lwd=2)
	abline(v=results[j,1],col="red",lwd=3,lty=2)
	curve(dnorm(x,mean=mean(betas.boot[,j]),sd=sd(betas.boot[,j])),add=TRUE,col="blue",lwd=2)
	mm=c("bootstrap","asymp. normal dist.")
	legend(dir[j],mm,lwd=2,col=c(1,"blue"),bty="n",cex=1.2)
	nombre<-paste("app3beta",j,".pdf",sep="")
	dev.copy(pdf,nombre)
	dev.off()
}


